@@ -8,7 +8,7 @@ from uuid import UUID
 from grpc import RpcError
 
 from preseem import NetworkMetadataReference
-from preseem.source_model import Attachment, Service
+from preseem.source_model import Attachment, Router, Service
 from preseem_protobuf.model.service_pb2 import SetAttachmentReq
 
 # STM-6039 Only allow the sync to run once at a time, to prevent duplicate
@@ -42,6 +42,13 @@ _package_field_map = {
     'upload_rate': lambda x: x.upload_rate or None,
 }
 
+_router_field_map = {
+    'name': lambda x: x.name,
+    'management_ip': lambda x: x.host,
+    'site_uuid': lambda x: x._site_uuid,
+    'intent_role_uuid': lambda _: ElementRole.ROUTER.bytes,
+}
+
 _service_field_map = {
     'account_uuid': lambda x: x._account_uuid,
     'package_uuid': lambda x: x._package_uuid,
@@ -171,6 +178,7 @@ async def sync_preseem_model(preseem_model,
                              packages=None,
                              accounts=None,
                              services=None,
+                             routers=None,
                              nm_model=None,
                              noact=False,
                              verbose=0):
@@ -193,10 +201,12 @@ async def sync_preseem_model(preseem_model,
         raise ValueError('source cannot be the empty string')
 
     async with sync_lock:
-        model_accounts, model_discs, model_services, model_sites, model_packages = await asyncio.gather(
+        model_accounts, model_discs, router_discs, model_services, model_sites, model_packages = await asyncio.gather(
             preseem_model.Account.List(company_uuid=company_uuid),
             preseem_model.Element.ListElementDiscovery(company_uuid=company_uuid,
                                                        intent_role_uuid=role.bytes),
+            preseem_model.Element.ListElementDiscovery(
+                company_uuid=company_uuid, intent_role_uuid=ElementRole.ROUTER.bytes),
             preseem_model.Service.List(company_uuid=company_uuid,
                                        include_attachment=True),
             preseem_model.Site.List(company_uuid=company_uuid),
@@ -444,4 +454,50 @@ async def sync_preseem_model(preseem_model,
                                 _service_field_map,
                                 noact=noact)
 
+        if routers is not None:
+            assert isinstance(routers, dict)
+            new_sites = set()
+            for router in routers.values():
+                assert isinstance(router, Router)
+                if not router.name:  # name is required for ElementDiscovery
+                    raise ValueError('invalid value for router name')
+                if not router.host:  # host is required for ElementDiscovery
+                    raise ValueError('invalid value for router host')
+                if router.site:
+                    site = site_names.get(router.site)
+                    if not site:
+                        new_sites.add(router.site)
+                    elif site.inactive:
+                        logging.info('sync_preseem_model(%s): reactivate site "%s"',
+                                     source, item.site)
+                        if not noact:
+                            await call_and_log_error(
+                                preseem_model.Site.Update(company_uuid=company_uuid,
+                                                          uuid=site.uuid,
+                                                          inactive=False))
+            for site_name in new_sites:
+                logging.info('sync_preseem_model(%s): create site "%s"', source,
+                             site_name)
+                if not noact:
+                    site = site_names[site_name] = await call_and_log_error(
+                        preseem_model.Site.Create(company_uuid=company_uuid,
+                                                  name=site_name))
+                    if site:
+                        model_sites[site.uuid] = site_names[site_name] = site
+
+            for router in routers.values():
+                if router.site:
+                    site = site_names.get(router.site)
+                    router._site_uuid = site.uuid
+
+            await _sync_objects(preseem_model,
+                                company_uuid,
+                                source,
+                                routers,
+                                router_discs,
+                                preseem_model.Element.CreateElementDiscovery,
+                                preseem_model.Element.UpdateElementDiscovery,
+                                _router_field_map,
+                                noact=noact)
+
         return uuid_map
