@@ -400,17 +400,21 @@ class Service(Errorable):
 
 class Router(Errorable):
     """Router class to use within a data source."""
-    __slots__ = ["_id", "_host", "_name", "_site", "_url"]
+    __slots__ = ["_id", "_community", "_host", "_name", "_site", "_url", "_site_uuid"]
+    # this is just used to list fields we want to print with __repr__
+    attrs = ('id', 'name', 'site', 'host', 'community', 'url')
 
     def __init__(self, id, **kwargs):
         super().__init__()
         if id is None or str(id) == '':
             raise ValueError("The 'id' parameter for a Router must be non-empty.")
         self._id = str(id)
+        self._community = None
         self._host = None
         self._name = None
         self._site = None
         self._url = None
+        self._site_uuid = None  # used by sync_preseem_model for resolution
         for key, value in kwargs.items():
             setattr(self, key, value)
 
@@ -418,6 +422,25 @@ class Router(Errorable):
     def id(self):
         return self._id
 
+    @property
+    def community(self):
+        return self._community
+
+    @community.setter
+    def community(self, value):
+        if value is not None:
+            if not isinstance(value, str):
+                self.err(
+                    'community', value,
+                    TypeError(
+                        "The router 'community' parameter must be of type string."))
+                return
+            if value == "":
+                self.err('community', value,
+                         "The router 'community' parameter must be non-empty.")
+                return
+        self._community = str(community) if value is not None else None
+
     @property
     def host(self):
         return self._host
@@ -482,3 +505,19 @@ class Router(Errorable):
                 self.err('url', value, "The router 'url' parameter must be non-empty.")
                 return
         self._url = value if value is not None else None
+
+    def __repr__(self):
+        attr_values = [(attr, getattr(self, attr)) for attr in self.attrs]
+        attributes_str = ', '.join(f"{attr}={value!r}" for attr, value in attr_values)
+        return f"{type(self).__name__}({attributes_str})"
+
+    def __eq__(self, other):
+        if isinstance(other, self.__class__):
+            for slot in self.__slots__:
+                if getattr(self, slot) != getattr(other, slot):
+                    return False
+            return True
+        return False
+
+    def __hash__(self):
+        return hash(self.id)
