@@ -11,7 +11,7 @@ from preseem import FakeNetworkMetadataModel, NetworkMetadataReference, FakePres
 from preseem.model_sync import sync_preseem_model, ElementRole, ElementSyncItem
 from preseem_protobuf.model.element_pb2 import ElementStatus
 from preseem_protobuf.model.account_pb2 import AccountType, AccountStatus
-from preseem.source_model import Attachment, Service
+from preseem.source_model import Attachment, Router, Service
 
 Account = namedtuple('Account', ('id', 'type', 'status', 'name', 'first_name', 'last_name'))
 Package = namedtuple('Package', ('id', 'name', 'download_rate', 'upload_rate'))
@@ -1146,3 +1146,109 @@ class TestModelSync(unittest.TestCase):
             await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], accounts=accounts, packages=packages, services=services)
             self.assertEqual(self.model._services[msvc.company_uuid][msvc.uuid].attachment.network_prefixes[0], "192.0.2.101/32")
         self._await(test())
+
+    async def checkSourceRouter(self, sobj):
+        sites = {}
+        for site in await self.model.Site.List(company_uuid=self.company.uuid):
+            if site.name in sites:
+                if isinstance(sites[site.name], list):
+                    sites[site.name].append(site)
+                else:
+                    sites[site.name] = [sites[site.name], site]
+            else:
+                sites[site.name] = site
+        routers = {x.source_id: x for x in await self.model.Element.ListElementDiscovery(company_uuid=self.company.uuid) if x.intent_role_uuid == ElementRole.ROUTER.bytes}
+        mobj = routers.get(sobj.id)
+        self.assertIsNotNone(mobj)
+        self.assertEqual(mobj.source_id, sobj.id)
+        self.assertEqual(mobj.name, sobj.name)
+        self.assertEqual(mobj.management_ip, sobj.host)
+        if sobj.site:
+            site = sites.get(sobj.site)
+            if site or not sobj.errors:
+                self.assertIsNotNone(site)
+                self.assertEqual(site.uuid, mobj.site_uuid)
+
+    def test_router_create(self):
+        """An Router is created in the model."""
+        async def test():
+            router = Router('router01', host='192.0.2.1', name='Router 1', site='Site 1', url='https://myisp.com/routers/router01')
+            routers = {'router01': router}
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], routers=routers)
+            await self.checkSourceRouter(router)
+        self._await(test())
+
+    def test_router_no_site(self):
+        """If a router does not specify a site, it is still created."""
+        async def test():
+            router = Router('router01', host='192.0.2.1', name='Router 1')
+            routers = {'router01': router}
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], routers=routers)
+            await self.checkSourceRouter(router)
+        self._await(test())
+
+    def test_router_update(self):
+        """A Router is updated in the model."""
+        async def test():
+            # create a router
+            router = Router('router01', host='192.0.2.1', name='Router 1', site='Site 1')
+            routers = {'router01': router}
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], routers=routers)
+            await self.checkSourceRouter(router)
+
+            # change attributes
+            router.name = 'Router 01'
+            router.host = '192.0.2.254'
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], routers=routers)
+            await self.checkSourceRouter(router)
+            print(await self.model.Element.ListElementDiscovery(company_uuid=self.company.uuid))
+        self._await(test())
+
+    def test_router_update(self):
+        """A Router is updated in the model."""
+        async def test():
+            # create a router
+            router = Router('router01', host='192.0.2.1', name='Router 1', site='Site 1')
+            routers = {'router01': router}
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], routers=routers)
+            await self.checkSourceRouter(router)
+
+            # change attributes
+            router.name = 'Router 01'
+            router.host = '192.0.2.254'
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], routers=routers)
+            await self.checkSourceRouter(router)
+            print(await self.model.Element.ListElementDiscovery(company_uuid=self.company.uuid))
+        self._await(test())
+
+    def test_router_update_error(self):
+        """Updating a field resolved by the syncer to an invalid value leaves the previous value."""
+        async def test():
+            # create a router
+            router = Router('router01', host='192.0.2.1', name='Router 1', site='Site 1')
+            routers = {'router01': router}
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], routers=routers)
+            await self.checkSourceRouter(router)
+
+            # test internal field checking
+            router.name = ''
+            self.assertEqual(router.name, 'Router 1')
+            self.assertTrue(router.errors)
+            router.errors.pop()
+            router.host = ''
+            self.assertEqual(router.host, '192.0.2.1')
+            self.assertTrue(router.errors)
+            router.errors.pop()
+
+            # test field checking done by sync
+            router.name = None
+            with self.assertRaises(ValueError):
+                await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], routers=routers)
+            router.host = None
+            with self.assertRaises(ValueError):
+                await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], routers=routers)
+
+            router.name = 'Router 1'
+            router.host = '192.0.2.1'
+            await self.checkSourceRouter(router)
+        self._await(test())
